/************************************************************************************
gpsinformationprocessingirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***********************************************************************************************//
//* main.cpp									     2009.12.14 *//
//*												*//
//* GTK+ モニタリングプログラム									*//
//***********************************************************************************************//


//-- [ヘッダファイル] ---------------------------------------------------------------------------//
#include <gtk/gtk.h>											// gtk/gtkヘッダファイル読み込み
#include <glade/glade.h>										// glade/gladeヘッダファイル読み込み
#include "udp.h"											// udpヘッダファイル読み込み
//-----------------------------------------------------------------------------------------------//

//-- [置換] -------------------------------------------------------------------------------------//
#define BUFSIZE		128										// 最大送受信文字数
#define DATASIZE	256										// 最大データ文字数
#define FONTSTYLE	"Liberation Sans"								// フォントスタイル
#define DEBUG_COMMAND_LINE										// DEBUG定義 [デバッグ用]
//-----------------------------------------------------------------------------------------------//


//-- [コールバック関数プロトタイプ宣言] ---------------------------------------------------------//
G_BEGIN_DECLS
void udp_read_callback(gpointer, gint, GdkInputCondition);						// UDPデータ受信イベント
void window_destroy(GtkWidget*);									// ウィンドウ破棄イベント
G_END_DECLS
//-----------------------------------------------------------------------------------------------//


//-- [変数宣言] ---------------------------------------------------------------------------------//
char send_data[BUFSIZE];										// 送信データ格納配列
char recv_data[BUFSIZE];										// 受信データ格納配列
int send_sock;												// ソケットディスクリプタ格納変数
int recv_sock;												// ソケットディスクリプタ格納変数
int recv_len;												// 受信サイズ格納変数
struct sockaddr_in send_addr;										// アドレス格納構造体
struct sockaddr_in recv_addr;										// アドレス格納構造体
socklen_t addr_size;											// アドレス格納構造体サイズ格納変数
GtkWidget *timestamp;											// ラベル [timestamp]
GtkWidget *latitude;											// ラベル [latitude]
GtkWidget *longitude;											// ラベル [longitude]
//-----------------------------------------------------------------------------------------------//


//-- [UDPデータ受信イベント] --------------------------------------------------------------------//
//- void udp_read_callback(gpointer data, gint source, GdkInputCondition condition)		-//
//- 機能  ：UDPのデータ受信処理を行う								-//
//- 引数  ：(gpointer)          関数引き渡しユーザデータ					-//
//-       ：(gint)              条件が発生するファイルディスクリプタ				-//
//-       ：(GdkInputCondition) トリガー条件							-//
//- 戻り値：(void)              なし								-//
//-----------------------------------------------------------------------------------------------//
void udp_read_callback(gpointer data, gint source, GdkInputCondition condition)
{


	//-- [変数宣言] -------------------------------------------------------------------------//

	int i;												// カウンタ格納変数
	int j;												// カウンタ格納変数
	int k;												// カウンタ格納変数
	char message[3][DATASIZE];									// メッセージ格納配列
	char label[3][DATASIZE];									// メッセージ格納配列

	//---------------------------------------------------------------------------------------//


	//-- [初期化] ---------------------------------------------------------------------------//

	for(i = 0; i < 3; i++) strcpy(message[i], "");							// メッセージ初期化

	//---------------------------------------------------------------------------------------//


	//-- [受信データクリア] -----------------------------------------------------------------//

	memset(recv_data, 0, sizeof(recv_data));							// バッファクリア

	//---------------------------------------------------------------------------------------//


	//-- [受信処理] -------------------------------------------------------------------------//

	recv_len = recvfrom(recv_sock, recv_data, sizeof(recv_data), 0,
		(struct sockaddr *)&recv_addr, &addr_size);						// データ受信

	if(recv_len < 0){

		printf("receive error!!\n");								// メッセージ表示
		fflush(stdout);										// バッファフラッシュ [printf表示用]

	}

	j = 0;												// カウンタリセット
	k = 0;												// カウンタリセット

	for(i = 0; i< recv_len; i++){

		if(recv_data[i] != ','){

			message[k][j] = recv_data[i];							// データ格納
			j++;										// カウント

		}else{

			message[k][j] = 0x00;								// 終端文字処理
			j = 0;										// カウンタリセット
			k++;										// カウンタ

		}

	}

	message[k][j] = 0x00;										// 終端文字処理

	//---------------------------------------------------------------------------------------//


	//-- [プログラム終了処理] ---------------------------------------------------------------//

	if(strcmp("quit", recv_data) == 0){

		close(send_sock);									// ソケットディスクリプタクローズ
		close(recv_sock);									// ソケットディスクリプタクローズ
		gtk_main_quit();									// メッセージループ回避

	//---------------------------------------------------------------------------------------//


	//-- [ラベル処理] -----------------------------------------------------------------------//

	}else{


		//-- [ラベル表示処理] -----------------------------------------------------------//

		strcpy(label[0], ": ");									// コロン
		strcat(label[0], message[0]);								// timestamp
		gtk_label_set_text((GtkLabel *)timestamp, label[0]);					// ラベル表示 [timestamp]

		strcpy(label[1], ": ");									// コロン
		strcat(label[1], message[1]);								// latitude
		gtk_label_set_text((GtkLabel *)latitude, label[1]);					// ラベル表示 [latitude]

		strcpy(label[2], ": ");									// コロン
		strcat(label[2], message[2]);								// longitude
		gtk_label_set_text((GtkLabel *)longitude, label[2]);					// ラベル表示 [longitude]

		//-------------------------------------------------------------------------------//


	}

	//---------------------------------------------------------------------------------------//


}
//-----------------------------------------------------------------------------------------------//


//-- [ウィンドウ破棄イベント] -------------------------------------------------------------------//
//- void window_destroy(GtkWidget *widget)							-//
//- 機能  ：ウィンドウ破棄イベントの処理を行う							-//
//- 引数  ：(GtkWidget*) 破棄するオブジェクト							-//
//- 戻り値：(void)       なし									-//
//-----------------------------------------------------------------------------------------------//
void window_destroy(GtkWidget *widget)
{


	//-- [プログラム終了処理] ---------------------------------------------------------------//

	close(send_sock);										// ソケットディスクリプタクローズ
	close(recv_sock);										// ソケットディスクリプタクローズ
	gtk_main_quit();										// メッセージループ回避

	//---------------------------------------------------------------------------------------//


}
//-----------------------------------------------------------------------------------------------//


//-- [メイン関数] -------------------------------------------------------------------------------//
//- int main(int argc, char *argv[])								-//
//- 機能  ：(int)    コマンドライン引数の数							-//
//- 引数  ：(char *) コマンドライン引数								-//
//- 戻り値：(int)    -										-//
//-----------------------------------------------------------------------------------------------//
int main(int argc, char *argv[])
{


	//-- [変数宣言] -------------------------------------------------------------------------//

	// UDPプログラム
	char ip[16];											// IPアドレス格納配列
	char send_port[6];										// 送信ポート番号格納配列
	char recv_port[6];										// 受信ポート番号格納配列

	// GUIプログラム
	char xml_addr[DATASIZE];									// XMLファイルアドレス格納配列
	GladeXML *xml;											// GladeXML
	GtkWidget *window;										// トップレベルウィンドウ
	GtkWidget *timestamplabel;									// ラベル [timestamplabel]
	GtkWidget *latitudelabel;									// ラベル [latitudelabel]
	GtkWidget *longitudelabel;									// ラベル [longitudelabel]

	//---------------------------------------------------------------------------------------//


	//-- [コマンドライン引数処理] -----------------------------------------------------------//
	// 第1引数:	managementofareamap							-//
	// 第2引数:	[IPアドレス]								-//
	// 第3引数:	[送信ポート番号]	gui    -> module				-//
	// 第4引数:	[受信ポート番号]	module -> gui					-//
	//---------------------------------------------------------------------------------------//

	if(argc != 4){

		printf("argument error!!\n");								// メッセージ表示
		fflush(stdout);										// バッファフラッシュ [printf表示用]
		exit(1);										// 強制終了

	}else{

		strcpy(xml_addr, argv[0]);								// GUIプログラムアドレス取得
		strcpy(ip, argv[1]);									// IPアドレス取得
		strcpy(send_port, argv[2]);								// 送信ポート番号取得
		strcpy(recv_port, argv[3]);								// 受信ポート番号取得


		//-- [デバッグcommand line] ----------------------------------------------------//

		#ifndef DEBUG_COMMAND_LINE								// デバッグ表示開始

		printf("\n[command line]\n");								// label
		printf("command      = %s\n", xml_addr);						// command
		printf("ip           = %s\n", ip);							// ip
		printf("send port    = %s\n", send_port);						// send_port
		printf("receive port = %s\n\n", recv_port);						// recv_port
		fflush(stdout);										// バッファフラッシュ [printf表示用]

		#endif											// デバッグ表示終了

		//------------------------------------------------------------------------------//

		strcat(xml_addr, ".glade");								// XMLファイルアドレス取得

	}

	//---------------------------------------------------------------------------------------//


	//-- [初期化] ---------------------------------------------------------------------------//

	memset(send_data, 0, sizeof(send_data));							// 送信データ初期化
	memset(recv_data, 0, sizeof(recv_data));							// 受信データ初期化

	//---------------------------------------------------------------------------------------//


	//-- [データ格納] -----------------------------------------------------------------------//

	strcpy(send_data, "success");									// 送信データ設定

	//---------------------------------------------------------------------------------------//


	//-- [ソケット作成] ---------------------------------------------------------------------//
	//-											-//
	//- プロトコル・ファミリ								-//
	//- AF_UNIX, AF_LOCAL	: ローカル通信							-//
	//- AF_INET		: IPv4 インターネットプロトコル					-//
	//- AF_INET6		: IPv6 インターネットプロトコル					-//
	//- AF_IPX		: IPX-Novellプロトコル						-//
	//- AF_NETLINK		: カーネルユーザデバイス					-//
	//- AF_X25		: ITU-T X.25/ISO-8208プロトコル					-//
	//- AF_AX25		: アマチュア無線AX.25プロトコル					-//
	//- AF_ATMPVC		: ATMPVC							-//
	//- AF_APPLETALK	: アップルトーク						-//
	//- AF_PACKET		: 低レベルパケットインターフェース				-//
	//-											-//
	//- 通信方式										-//
	//- SOCK_STREAM		: バイトストリーム	[信頼性有:順序保証有:双方向]		-//
	//- SOCK_DGRAM		: データグラム		[信頼性無:固定最大長メッセージ]		-//
	//- SOCK_SEQPACKET	: データグラム		[信頼性有:固定最大長メッセージ:双方向]	-//
	//- SOCK_RAW		: 生ネットワーク						-//
	//- SOCK_RDM		: データグラム		[信頼性有:順序保証無]			-//
	//- SOCK_PACKET		: 廃止								-//
	//---------------------------------------------------------------------------------------//

	if((send_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){						// ソケット作成 [送信データ用]

		printf("socket error!!\n");								// メッセージ表示
		fflush(stdout);										// バッファフラッシュ [printf表示用]
		exit(1);										// 強制終了

	}

	if((recv_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){						// ソケット作成 [受信データ用]

		printf("socket error!!\n");								// メッセージ表示
		fflush(stdout);										// バッファフラッシュ [printf表示用]
		exit(1);										// 強制終了

	}

	//---------------------------------------------------------------------------------------//


	//-- [アドレス設定] ---------------------------------------------------------------------//

	send_addr.sin_family      = AF_INET;								// アドレスファミリ
	send_addr.sin_port        = htons(atoi(send_port));						// ポート番号 [バイトオーダ]
	send_addr.sin_addr.s_addr = inet_addr(ip);							// IPアドレス [バイトオーダ]

	recv_addr.sin_family      = AF_INET;								// アドレスファミリ
	recv_addr.sin_port        = htons(atoi(recv_port));						// ポート番号 [バイトオーダ]
	recv_addr.sin_addr.s_addr = inet_addr(ip);							// IPアドレス [バイトオーダ]

	//---------------------------------------------------------------------------------------//


	//-- [受信設定] -------------------------------------------------------------------------//

	bind(recv_sock, (struct sockaddr *)&recv_addr, sizeof(recv_addr));				// バインド [受信データ]
	addr_size = sizeof(recv_addr);									// アドレス格納構造体サイズ格納

	//---------------------------------------------------------------------------------------//


	//-- [GTK+初期化] -----------------------------------------------------------------------//

	gtk_init(&argc, &argv);										// GTK+初期化

	if(!(xml = glade_xml_new(xml_addr, NULL, NULL))){						// データ読み込み

		printf("glade file error.\n");								// メッセージ表示
		return 1;										// 戻り値 [異常終了]

	}


	//-- [ウィジェット生成] -----------------------------------------------------------------//

	window         = glade_xml_get_widget(xml, "window");						// ウィンドウ生成
	timestamplabel = glade_xml_get_widget(xml, "timestamplabel");					// ラベル生成 [timestamplabel]
	latitudelabel  = glade_xml_get_widget(xml, "latitudelabel");					// ラベル生成 [latitudelabel]
	longitudelabel = glade_xml_get_widget(xml, "longitudelabel");					// ラベル生成 [longitudelabel]
	timestamp      = glade_xml_get_widget(xml, "timestamp");					// ラベル生成 [timestamp]
	latitude       = glade_xml_get_widget(xml, "latitude");						// ラベル生成 [latitude]
	longitude      = glade_xml_get_widget(xml, "longitude");					// ラベル生成 [longitude]

	//---------------------------------------------------------------------------------------//


	//-- [フォント設定] ---------------------------------------------------------------------//

	gtk_widget_modify_font(timestamplabel, pango_font_description_from_string(FONTSTYLE));		// フォント設定 [timestamplabel]
	gtk_widget_modify_font(latitudelabel, pango_font_description_from_string(FONTSTYLE));		// フォント設定 [latitudelabel]
	gtk_widget_modify_font(longitudelabel, pango_font_description_from_string(FONTSTYLE));		// フォント設定 [longitudelabel]
	gtk_widget_modify_font(timestamp, pango_font_description_from_string(FONTSTYLE));		// フォント設定 [timestamp]
	gtk_widget_modify_font(latitude, pango_font_description_from_string(FONTSTYLE));		// フォント設定 [latitude]
	gtk_widget_modify_font(longitude, pango_font_description_from_string(FONTSTYLE));		// フォント設定 [longitude]

	//---------------------------------------------------------------------------------------//


	glade_xml_signal_autoconnect(xml);								// コールバック関数接続
	gtk_widget_show(window);									// ウィンドウ表示

	//---------------------------------------------------------------------------------------//


	//-- [ソケットディスクリプタ登録] -------------------------------------------------------//

	gdk_input_add(recv_sock, GDK_INPUT_READ, udp_read_callback, NULL);				// コールバック関数登録

	//---------------------------------------------------------------------------------------//


	//-- [イベントループ] -------------------------------------------------------------------//

	gtk_main();											// イベントループ

	//---------------------------------------------------------------------------------------//


	return 0;											// 戻り値 [正常終了]

}
//-----------------------------------------------------------------------------------------------//
