/************************************************************************************
gpsinformationprocessingirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

#ifndef GPSINFORMATIONPROCESSING_H
#define GPSINFORMATIONPROCESSING_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>


//-- [ヘッダファイル] ---------------------------------------------------------------------------//
#include "legacyinfo.h"											// legacyinfoヘッダファイル読み込み
#include <math.h>											// mathヘッダファイル読み込み
//-----------------------------------------------------------------------------------------------//


//-- [置換] -------------------------------------------------------------------------------------//
#define WAITTIME	1000000										// 起動待ち時間 [us]
#define DATASIZE	16										// 最大データ文字数
#define DEBUG_CONFIGURATION										// DEBUG定義 [デバッグ用]
#define DEBUG_RECV											// DEBUG定義 [デバッグ用]
#define DEBUG_INPORT											// DEBUG定義 [デバッグ用]
#define DEBUG_GGA											// DEBUG定義 [デバッグ用]
#define DEBUG_GLL											// DEBUG定義 [デバッグ用]
#define DEBUG_GNS											// DEBUG定義 [デバッグ用]
#define DEBUG_GRS											// DEBUG定義 [デバッグ用]
#define DEBUG_GSA											// DEBUG定義 [デバッグ用]
#define DEBUG_GST											// DEBUG定義 [デバッグ用]
#define DEBUG_GSV											// DEBUG定義 [デバッグ用]
#define DEBUG_HDT											// DEBUG定義 [デバッグ用]
#define DEBUG_RMC											// DEBUG定義 [デバッグ用]
#define DEBUG_VTG											// DEBUG定義 [デバッグ用]
#define DEBUG_ZDA											// DEBUG定義 [デバッグ用]
//-----------------------------------------------------------------------------------------------//


// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "intellirobotStub.h"

// </rtc-template>

using namespace RTC;

class GpsInformationProcessing
  : public RTC::DataFlowComponentBase
{
 public:
  GpsInformationProcessing(RTC::Manager* manager);
  ~GpsInformationProcessing();

	// The initialize action (on CREATED->ALIVE transition)
	// formaer rtc_init_entry()
	virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

	// The activated action (Active state entry action)
	// former rtc_active_entry()
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	// The deactivated action (Active state exit action)
	// former rtc_active_exit()
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	// The execution action that is invoked periodically
	// former rtc_active_do()
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  std::string m_01_gui;
  std::string m_02_guiaddress;
  std::string m_03_ip;
  std::string m_04_sendport;
  std::string m_05_receiveport;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedLegacyData m_LegacyData;
  InPort<TimedLegacyData> m_LegacyDataIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedJgd2000Position m_Jgd2000Position;
  OutPort<TimedJgd2000Position> m_Jgd2000PositionOut;
  TimedGpsData m_GpsData;
  OutPort<TimedGpsData> m_GpsDataOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

	private:
		int dummy;


		//-- [変数宣言] -----------------------------------------------------------------//

		// 情報処理プログラム
		int i;											// カウンタ格納変数
		int gui_flag;										// GUI動作フラグ格納変数
		int mode;										// メッセージモード格納変数
		unsigned long sec;									// タイムスタンプ格納変数 [sec]
		unsigned long nsec;									// タイムスタンプ格納変数 [nsec]
		char gui[4];										// GUI動作格納配列

		// 入力ポート
		char gga[16][DATASIZE];									// GGAメッセージ格納配列
		char gll[9][DATASIZE];									// GLLメッセージ格納配列
		char gns[13][DATASIZE];									// GNSメッセージ格納配列 [GNGNS]
		char gp_gns[13][DATASIZE];								// GNSメッセージ格納配列 [GPGNS]
		char gl_gns[13][DATASIZE];								// GNSメッセージ格納配列 [GLGNS]
		char gp_grs[16][DATASIZE];								// GRSメッセージ格納配列 [GPGRS]
		char gl_grs[16][DATASIZE];								// GRSメッセージ格納配列 [GLGRS]
		char gp_gsa[19][DATASIZE];								// GSAメッセージ格納配列 [GPGSA]
		char gl_gsa[19][DATASIZE];								// GSAメッセージ格納配列 [GLGSA]
		char gst[10][DATASIZE];									// GSTメッセージ格納配列
		char gp_gsv[3][21][DATASIZE];								// GSVメッセージ格納配列 [GPGSV]
		char gl_gsv[3][21][DATASIZE];								// GSVメッセージ格納配列 [GLGSV]
		char hdt[4][DATASIZE];									// HDTメッセージ格納配列
		char rmc[14][DATASIZE];									// RMCメッセージ格納配列
		char vtg[11][DATASIZE];									// VTGメッセージ格納配列
		char zda[8][DATASIZE];									// ZDAメッセージ格納配列

		// 出力ポート
		unsigned long gga_sec;									// タイムスタンプ格納変数 [sec]
		unsigned long gga_nsec;									// タイムスタンプ格納変数 [nsec]
		unsigned long gll_sec;									// タイムスタンプ格納変数 [sec]
		unsigned long gll_nsec;									// タイムスタンプ格納変数 [nsec]
		unsigned long gns_sec;									// タイムスタンプ格納変数 [sec]
		unsigned long gns_nsec;									// タイムスタンプ格納変数 [nsec]
		unsigned long gp_gns_sec;								// タイムスタンプ格納変数 [sec]
		unsigned long gp_gns_nsec;								// タイムスタンプ格納変数 [nsec]
		unsigned long gl_gns_sec;								// タイムスタンプ格納変数 [sec]
		unsigned long gl_gns_nsec;								// タイムスタンプ格納変数 [nsec]
		unsigned long gp_grs_sec;								// タイムスタンプ格納変数 [sec]
		unsigned long gp_grs_nsec;								// タイムスタンプ格納変数 [nsec]
		unsigned long gl_grs_sec;								// タイムスタンプ格納変数 [sec]
		unsigned long gl_grs_nsec;								// タイムスタンプ格納変数 [nsec]
		unsigned long gst_sec;									// タイムスタンプ格納変数 [sec]
		unsigned long gst_nsec;									// タイムスタンプ格納変数 [nsec]
		unsigned long rmc_sec;									// タイムスタンプ格納変数 [sec]
		unsigned long rmc_nsec;									// タイムスタンプ格納変数 [nsec]
		unsigned long zda_sec;									// タイムスタンプ格納変数 [sec]
		unsigned long zda_nsec;									// タイムスタンプ格納変数 [nsec]
		double latitude;									// 緯度格納変数
		double longitude;									// 経度格納変数

		// GUIプログラム
		char gui_addr[256];									// GUIプログラムアドレス格納配列
		char ip[16];										// IPアドレス格納配列
		char send_port[6];									// 送信ポート番号格納配列
		char recv_port[6];									// 受信ポート番号格納配列
		char command[256];									// コマンドライン引数格納配列

		// UDP通信
		double send_value[3];									// 送信元データ格納配列
		char send_data[BUFSIZE];								// 送信データ格納配列
		char recv_data[BUFSIZE];								// 受信データ格納配列
		int send_sock;										// ソケットディスクリプタ格納変数
		int recv_sock;										// ソケットディスクリプタ格納変数
		int recv_len;										// 受信サイズ格納変数
		struct sockaddr_in send_addr;								// アドレス格納構造体
		struct sockaddr_in recv_addr;								// アドレス格納構造体
		struct sigaction saio;									// シグナル受信時動作設定格納構造体
		socklen_t addr_size;									// アドレス格納構造体サイズ格納変数

		//-------------------------------------------------------------------------------//


};


extern "C"
{
  void GpsInformationProcessingInit(RTC::Manager* manager);
};

#endif // GPSINFORMATIONPROCESSING_H
